/*
 * Iface.java
 *
 * Created on 13 de octubre de 2008, 16:07
 */
package com.imaxin.exeria;

import com.imaxin.exeria.utilities.EngineConnector;
import com.imaxin.exeria.utilities.XeCore;
import com.imaxin.exeria.utilities.NSError;
import com.imaxin.exeria.utilities.TextProcessor;
import com.sun.star.text.XTextDocument;
import com.imaxin.exeria.utilities.PanelLogo;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

/**
 *
 * @author  stefanos
 */
public class Iface extends javax.swing.JFrame implements Runnable {

    java.net.URL imgBackground = this.getClass().getClassLoader().getResource("com/imaxin/exeria/img/logo_no_text.png");
    TextProcessor tp;
    NSError error;
    List ignoredSentences = new ArrayList();
    ResourceBundle bundle;
    public static boolean flagEndThread = false;
    private int countTimes;
    private static final int MAX_ERROR_TIMES = 5;  
    private Object[] startOptions;

    /** Creates new form Iface */
    public Iface(XTextDocument xTextDoc) {
        countTimes = 0;
        flagEndThread = false;
        XeCore.restartService();
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            System.out.println("e = " + e);
        }
        tp = new TextProcessor(xTextDoc);
        initComponents();
        chkAutoCorrect.setVisible(false);
        bundle = ResourceBundle.getBundle("com/imaxin/exeria/resources/exeria");
        startOptions = new Object[]{bundle.getString("confirmDialog.response.yes"), bundle.getString("confirmDialog.response.no")};
    }

    private void beginCorrection() {
        new Thread(this).start();
    }

    private void checkSentence() {        
        if (!flagEndThread) {
            preCheck();
            //Get the next sentence
            String nextSentence = tp.getNextSentence();

            if (nextSentence != null) {
                if (ignoredSentences.contains(nextSentence)) {
                    //Sentence will be ignored
                    checkSentence();
                } else {                                        
                    try {
                        //Get responce from Exeria Engine
                        String response = EngineConnector.checkText(nextSentence);

                        if (!response.equals("")) {
                            countTimes = 0;
                            //There is an error
                            postCheck(nextSentence, response);
                        } else {
                            //No error
                            countTimes = 0;
                            checkSentence();
                        }

                    } catch (Exception e) {
                        System.out.println("response = " + nextSentence);
                        if (countTimes == MAX_ERROR_TIMES) {
                            JOptionPane.showMessageDialog(this, bundle.getString("confirmDialog.connection_error"));
                            exitExeria();
                            countTimes = 0;
                        } else {
                            XeCore.restartService();
                            countTimes++;
                            tp.reSelectSentence();
                            checkSentence();
                        }
                    }

                }
            } else {
                //Ask user wether to start over
                startOver();
            }
        }
    }

    private void exitExeria() {
        flagEndThread = true;
        XeCore.kill();
        Main.ifaceOpen = false;
        this.dispose();
    }

    private void postCheck(String originalSentence, String response) {

        error = new NSError(originalSentence, response);

        if (error.canCorrect()) {

            barProgress.setIndeterminate(false);
            lstSuggestion.setListData(error.getSuggestions());
            txtSubstitute.setText(error.getSuggestions()[0]);

            if (!chkAutoCorrect.isSelected()) {
                btoStop.setEnabled(false);
                btoIgnore.setEnabled(true);
                btoIgnoreAll.setEnabled(true);
                btoReplace.setEnabled(true);
                btoReplaceNStop.setEnabled(true);
                txtCurrentSentence.setText(error.getOriginalSentence());
                txtCurrentSentence.requestFocus();
                txtCurrentSentence.setSelectionStart(error.getSelectionStart());
                txtCurrentSentence.setSelectionEnd(error.getSelectionEnd());
            } else {
                //Autocorrect fires Action Replace
                btoReplaceActionPerformed(null);
            }

        } else {
            checkSentence();
        }
    }

    private void preCheck() {
        barProgress.setIndeterminate(true);
        btoIgnore.setEnabled(false);
        btoIgnoreAll.setEnabled(false);
        btoReplace.setEnabled(false);
        btoReplaceNStop.setEnabled(false);
        btoStop.setEnabled(true);
        txtCurrentSentence.setText("");
        txtSubstitute.setText("");
        lstSuggestion.setListData(new String[0]);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new PanelLogo(new ImageIcon(imgBackground).getImage());
        jScrollPane1 = new javax.swing.JScrollPane();
        lstSuggestion = new javax.swing.JList();
        txtCurrentSentence = new javax.swing.JTextField();
        btoReplace = new javax.swing.JButton();
        btoIgnore = new javax.swing.JButton();
        btoStart = new javax.swing.JButton();
        barProgress = new javax.swing.JProgressBar();
        btoIgnoreAll = new javax.swing.JButton();
        chkAutoCorrect = new javax.swing.JCheckBox();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        txtSubstitute = new javax.swing.JTextField();
        btoReplaceNStop = new javax.swing.JButton();
        btoContinue = new javax.swing.JButton();
        btoStop = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("com/imaxin/exeria/resources/exeria"); // NOI18N
        setTitle(bundle.getString("frmTitle.text")); // NOI18N
        setAlwaysOnTop(true);
        setBackground(new java.awt.Color(255, 255, 255));
        setIconImage(new ImageIcon(getClass().getResource("/com/imaxin/exeria/img/logo32.png")).getImage());
        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jPanel1.setBackground(new java.awt.Color(255, 255, 255));

        lstSuggestion.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        lstSuggestion.setSelectedIndex(0);
        lstSuggestion.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                lstSuggestionValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(lstSuggestion);

        txtCurrentSentence.setEditable(false);
        txtCurrentSentence.setFont(new java.awt.Font("Tahoma", 1, 11));
        txtCurrentSentence.setSelectionColor(new java.awt.Color(245, 104, 132));

        btoReplace.setText(bundle.getString("btoReplace.text")); // NOI18N
        btoReplace.setEnabled(false);
        btoReplace.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btoReplaceActionPerformed(evt);
            }
        });

        btoIgnore.setText(bundle.getString("btoIgnore.text")); // NOI18N
        btoIgnore.setEnabled(false);
        btoIgnore.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btoIgnoreActionPerformed(evt);
            }
        });

        btoStart.setText(bundle.getString("btoStart.text")); // NOI18N
        btoStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btoStartActionPerformed(evt);
            }
        });

        barProgress.setOpaque(true);

        btoIgnoreAll.setText(bundle.getString("btoIgnoreAll.text")); // NOI18N
        btoIgnoreAll.setEnabled(false);
        btoIgnoreAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btoIgnoreAllActionPerformed(evt);
            }
        });

        chkAutoCorrect.setBackground(new java.awt.Color(255, 255, 255));
        chkAutoCorrect.setText(bundle.getString("chkAutoCorrect.text")); // NOI18N

        jLabel1.setText(bundle.getString("lblOriginal.text")); // NOI18N

        jLabel2.setText(bundle.getString("lblSuggestions.text")); // NOI18N

        jLabel3.setText(bundle.getString("lblSubstitute.text")); // NOI18N

        btoReplaceNStop.setText(bundle.getString("btoReplaceNStop.text")); // NOI18N
        btoReplaceNStop.setEnabled(false);
        btoReplaceNStop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btoReplaceNStopActionPerformed(evt);
            }
        });

        btoContinue.setText(bundle.getString("btoContinue.text")); // NOI18N
        btoContinue.setEnabled(false);
        btoContinue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btoContinueActionPerformed(evt);
            }
        });

        btoStop.setText(bundle.getString("btoStop.text")); // NOI18N
        btoStop.setEnabled(false);
        btoStop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btoStopActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, barProgress, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 523, Short.MAX_VALUE)
                    .add(chkAutoCorrect)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanel1Layout.createSequentialGroup()
                                .add(jLabel3)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED))
                            .add(jPanel1Layout.createSequentialGroup()
                                .add(jLabel2)
                                .add(241, 241, 241))
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 378, Short.MAX_VALUE)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, jLabel1)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, txtCurrentSentence, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 378, Short.MAX_VALUE)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, txtSubstitute, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 378, Short.MAX_VALUE))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)))
                        .add(18, 18, 18)
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(btoStop, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(btoContinue, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(btoIgnoreAll, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(btoIgnore, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 107, Short.MAX_VALUE)
                            .add(btoStart, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 107, Short.MAX_VALUE)
                            .add(btoReplace, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 107, Short.MAX_VALUE)
                            .add(btoReplaceNStop, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .add(4, 4, 4)))
                .addContainerGap())
        );

        jPanel1Layout.linkSize(new java.awt.Component[] {btoIgnore, btoIgnoreAll, btoReplace, btoReplaceNStop, btoStart}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(txtCurrentSentence, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(jLabel3)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(txtSubstitute, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 81, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(chkAutoCorrect))
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(btoStart)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btoIgnore)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btoIgnoreAll)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btoReplace)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btoReplaceNStop)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btoContinue)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btoStop)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(barProgress, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void btoStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btoStartActionPerformed
        
    btoStart.setEnabled(false);
    if( 0 == JOptionPane.showOptionDialog(this, bundle.getString("confirmDialog.startPoint"), bundle.getString("btoStart.text"), JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null, startOptions, startOptions[1])){
        tp.goToStart();
    } else {
        tp.goToCurrentSentence();
    }
    beginCorrection();
}//GEN-LAST:event_btoStartActionPerformed

private void btoReplaceActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btoReplaceActionPerformed
    tp.reSelectSentence();
    tp.rewriteSentence(error.getOriginalSentence(), error.getNewSentence(txtSubstitute.getText()));
    beginCorrection();
}//GEN-LAST:event_btoReplaceActionPerformed

private void btoIgnoreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btoIgnoreActionPerformed
    beginCorrection();
}//GEN-LAST:event_btoIgnoreActionPerformed

private void btoIgnoreAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btoIgnoreAllActionPerformed
    ignoredSentences.add(error.getOriginalSentence());
    beginCorrection();
}//GEN-LAST:event_btoIgnoreAllActionPerformed

private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
    exitExeria();
}//GEN-LAST:event_formWindowClosing

private void lstSuggestionValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_lstSuggestionValueChanged
    String selection = (String) lstSuggestion.getSelectedValue();
    if (selection != null) {
        txtSubstitute.setText(selection);
    }
}//GEN-LAST:event_lstSuggestionValueChanged

private void btoReplaceNStopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btoReplaceNStopActionPerformed
    tp.reSelectSentence();
    tp.rewriteSentence(error.getOriginalSentence(), error.getNewSentence(txtSubstitute.getText()));
    btoIgnore.setEnabled(false);
    btoIgnoreAll.setEnabled(false);
    btoReplace.setEnabled(false);
    btoReplaceNStop.setEnabled(false);
    btoStart.setEnabled(false);
    btoContinue.setEnabled(true);
}//GEN-LAST:event_btoReplaceNStopActionPerformed

private void btoContinueActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btoContinueActionPerformed
    btoContinue.setEnabled(false);
    flagEndThread = false;
    beginCorrection();
}//GEN-LAST:event_btoContinueActionPerformed

private void btoStopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btoStopActionPerformed
    barProgress.setIndeterminate(false);
    flagEndThread = true;
    btoIgnore.setEnabled(false);
    btoIgnoreAll.setEnabled(false);
    btoReplace.setEnabled(false);
    btoReplaceNStop.setEnabled(false);
    btoStart.setEnabled(false);
    btoStop.setEnabled(false);
    btoContinue.setEnabled(true);    
}//GEN-LAST:event_btoStopActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JProgressBar barProgress;
    private javax.swing.JButton btoContinue;
    private javax.swing.JButton btoIgnore;
    private javax.swing.JButton btoIgnoreAll;
    private javax.swing.JButton btoReplace;
    private javax.swing.JButton btoReplaceNStop;
    private javax.swing.JButton btoStart;
    private javax.swing.JButton btoStop;
    private javax.swing.JCheckBox chkAutoCorrect;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JList lstSuggestion;
    private javax.swing.JTextField txtCurrentSentence;
    private javax.swing.JTextField txtSubstitute;
    // End of variables declaration//GEN-END:variables
    public void run() {
        try {
            checkSentence();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void startOver() {
        barProgress.setIndeterminate(false);
        if( 0 == JOptionPane.showOptionDialog(this, bundle.getString("confirmDialog.restart"), bundle.getString("btoStart.text"), JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null, startOptions, startOptions[1])){        
            tp.goToStart();
            ignoredSentences.removeAll(ignoredSentences);
            checkSentence();
        } else {
            exitExeria();
        }
    }
}
